@@ -120,7 +120,7 @@ class URI(object):
 
 
 class _RemoteMethod(object):
-    """method call abstraction, adapted from Python's xmlrpclib, but without nested calls at the moment"""
+    """method call abstraction"""
     def __init__(self, send, name):
         self.__send = send
         self.__name = name
@@ -131,14 +131,14 @@ class _RemoteMethod(object):
     def __call__(self, *args, **kwargs):
         return self.__send(self.__name, args, kwargs)
 
-
 def _check_hmac():
     if Pyro4.config.HMAC_KEY is None or len(Pyro4.config.HMAC_KEY)==0:
         import warnings
         warnings.warn("HMAC_KEY not set, protocol data may not be secure")
-    if sys.version_info>=(3,0) and type(Pyro4.config.HMAC_KEY) is not bytes:
+    elif sys.version_info>=(3,0) and type(Pyro4.config.HMAC_KEY) is not bytes:
         raise errors.PyroError("HMAC_KEY must be bytes type")
 
+
 class Proxy(object):
     """Pyro proxy for a remote object. Intercepts method calls and dispatches them to the remote object."""
     _pyroSerializer=util.Serializer()
@@ -214,7 +214,7 @@ class Proxy(object):
             self._pyroConnection.timeout=timeout
     _pyroTimeout=property(__pyroGetTimeout, __pyroSetTimeout)
 
-    def __pyroInvoke(self, methodname, vargs, kwargs):
+    def __pyroInvoke(self, methodname, vargs, kwargs, flags=0):
         """perform the remote method call communication"""
         if not self._pyroConnection:
             # rebind here, don't do it from inside the invoke because deadlock will occur
@@ -222,7 +222,6 @@ class Proxy(object):
         data, compressed=self._pyroSerializer.serialize(
             (self._pyroConnection.objectId, methodname, vargs, kwargs),
             compress=Pyro4.config.COMPRESSION)
-        flags=0
         if compressed:
             flags |= MessageFactory.FLAGS_COMPRESSED
         if methodname in self._pyroOneway:
@@ -287,6 +286,8 @@ class Proxy(object):
                 if msgType==MessageFactory.MSG_CONNECTFAIL:
                     error="connection rejected"
                     if data:
+                        if sys.version_info>=(3,0):
+                            data=str(data,"utf-8")
                         error+=", reason: "+data
                     conn.close()
                     log.error(error)
@@ -319,6 +320,67 @@ class Proxy(object):
         log.error(msg)
         raise errors.ConnectionClosedError(msg)
 
+    def _pyroBatch(self):
+        return _BatchProxy(self.__pyroInvokeBatch)
+
+    def __pyroInvokeBatch(self, calls, oneway=False):
+        flags=MessageFactory.FLAGS_BATCH
+        if oneway:
+            flags|=MessageFactory.FLAGS_ONEWAY
+        return self.__pyroInvoke("<batch>", calls, None, flags)
+
+
+class _BatchedRemoteMethod(object):
+    """method call abstraction that is used with batched calls"""
+    def __init__(self, calls, name):
+        self.__calls = calls
+        self.__name = name
+
+    def __getattr__(self, name):
+        return _BatchedRemoteMethod(self.__calls, "%s.%s" % (self.__name, name))
+
+    def __call__(self, *args, **kwargs):
+        self.__calls.append((self.__name, args, kwargs))
+
+
+class _BatchProxy(object):
+    """Helper class that lets you batch multiple method calls into one.
+    It is constructed with a reference to the method of the normal proxy that will
+    carry out the batched calls. Call methods on this object thatyou want to batch,
+    and finally call the batch proxy itself. That call will return a generator
+    for the results of every method call in the batch (in sequence)."""
+    def __init__(self, batchmethod):
+        self.__batchmethod=batchmethod
+        self.__calls=[]
+
+    def __getattr__(self, name):
+        return _BatchedRemoteMethod(self.__calls, name)
+
+    def __resultsgenerator(self, results):
+        for result in results:
+            if isinstance(result, _ExceptionWrapper):
+                result.raiseIt()   # re-raise the remote exception locally.
+            else:
+                yield result   # it is a regular result object, yield that and continue.
+
+    def __call__(self, oneway=False):
+        results=self.__batchmethod(self.__calls, oneway)
+        self.__calls=[]   # clear for re-use
+        if not oneway:
+            return self.__resultsgenerator(results)
+
+
+class _ExceptionWrapper(object):
+    """Class that wraps a remote exception. If this is returned, Pyro will
+    re-throw the exception on the receiving side. Usually this is taken care of
+    by a special response message flag, but in the case of batched calls this
+    flag is useless and another mechanism was needed."""
+    def __init__(self, exception):
+        self.exception=exception
+
+    def raiseIt(self):
+        raise self.exception
+
 
 class MessageFactory(object):
     """internal helper class to construct Pyro protocol messages"""
@@ -341,6 +403,7 @@ class MessageFactory(object):
     FLAGS_COMPRESSED = 1<<1
     FLAGS_ONEWAY = 1<<2
     FLAGS_HMAC = 1<<3
+    FLAGS_BATCH = 1<<4
     MAGIC = 0x34E9
     if sys.version_info>=(3, 0):
         empty_bytes = bytes([])
@@ -485,7 +548,10 @@ class Daemon(object):
         # For now, client is not sending anything. Just respond with a CONNECT_OK.
         # We need a minimal amount of data or the socket will remain blocked
         # on some systems... (messages smaller than 40 bytes)
-        data,_=self.serializer.serialize("ok",compress=False)
+        # Return True for successful handshake, False if something was wrong.
+        data="ok"
+        if sys.version_info>=(3,0):
+            data=bytes(data,"utf-8")
         msg=MessageFactory.createMessage(MessageFactory.MSG_CONNECTOK, data, 0, 1)
         conn.send(msg)
         return True
@@ -498,6 +564,7 @@ class Daemon(object):
         """
         flags=0
         seq=0
+        wasBatched=False
         isCallback=False
         try:
             msgType, flags, seq, data = MessageFactory.getMessage(conn, MessageFactory.MSG_INVOKE)
@@ -508,16 +575,33 @@ class Daemon(object):
                 if kwargs and sys.version_info<(2, 6, 5) and os.name!="java":
                     # Python before 2.6.5 doesn't accept unicode keyword arguments
                     kwargs = dict((str(k), kwargs[k]) for k in kwargs)
-                #log.debug("calling %s.%s", obj.__class__.__name__, method)
-                obj=util.resolveDottedAttribute(obj, method, Pyro4.config.DOTTEDNAMES)
-                if flags & MessageFactory.FLAGS_ONEWAY and Pyro4.config.ONEWAY_THREADED:
-                    # oneway call to be run inside its own thread
-                    thread=threadutil.Thread(target=obj, args=vargs, kwargs=kwargs)
-                    thread.setDaemon(True)
-                    thread.start()
+                if flags & MessageFactory.FLAGS_BATCH:
+                    # batched method calls, loop over them all and collect all results
+                    data=[]
+                    for method,vargs,kwargs in vargs:
+                        method=util.resolveDottedAttribute(obj, method, Pyro4.config.DOTTEDNAMES)
+                        try:
+                            result=method(*vargs, **kwargs)   # this is the actual method call to the Pyro object
+                        except Exception:
+                            xt,xv=sys.exc_info()[0:2]
+                            log.debug("Exception occurred while handling batched request: %s", xv)
+                            xv._pyroTraceback=util.formatTraceback(detailed=Pyro4.config.DETAILED_TRACEBACK)
+                            data.append(_ExceptionWrapper(xv))
+                            break   # stop processing the rest of the batch
+                        else:
+                            data.append(result)
+                    wasBatched=True
                 else:
-                    isCallback=getattr(obj, "_pyroCallback", False)
-                    data=obj(*vargs, **kwargs)   # this is the actual method call to the Pyro object
+                    # normal single method call
+                    method=util.resolveDottedAttribute(obj, method, Pyro4.config.DOTTEDNAMES)
+                    if flags & MessageFactory.FLAGS_ONEWAY and Pyro4.config.ONEWAY_THREADED:
+                        # oneway call to be run inside its own thread
+                        thread=threadutil.Thread(target=method, args=vargs, kwargs=kwargs)
+                        thread.setDaemon(True)
+                        thread.start()
+                    else:
+                        isCallback=getattr(method, "_pyroCallback", False)
+                        data=method(*vargs, **kwargs)   # this is the actual method call to the Pyro object
             else:
                 log.debug("unknown object requested: %s", objId)
                 raise errors.DaemonError("unknown object")
@@ -528,6 +612,8 @@ class Daemon(object):
                 flags=0
                 if compressed:
                     flags |= MessageFactory.FLAGS_COMPRESSED
+                if wasBatched:
+                    flags |= MessageFactory.FLAGS_BATCH
                 msg=MessageFactory.createMessage(MessageFactory.MSG_RESULT, data, flags, seq)
                 del data
                 conn.send(msg)
