@@ -8,11 +8,17 @@ irmen@razorvine.net - http://www.razorvine.net/python/Pyro
 from __future__ import with_statement
 import unittest
 import copy
+import logging
+import os, sys
+import Pyro4.configuration
 import Pyro4.core
 import Pyro4.errors
 import Pyro4.constants
 from testsupport import *
 
+if sys.version_info>=(3,0):
+    import imp
+    reload=imp.reload
 
 class Thing(object):
     def __init__(self, arg):
@@ -21,6 +27,19 @@ class Thing(object):
         return self.arg==other.arg
     __hash__=object.__hash__
 
+
+class CoreTestsWithoutHmac(unittest.TestCase):
+    def testProxy(self):
+        Pyro4.config.HMAC_KEY=None
+        # check that proxy without hmac is possible
+        _=Pyro4.Proxy("PYRO:object@host:9999")
+    def testDaemon(self):
+        Pyro4.config.HMAC_KEY=None
+        # check that daemon without hmac is possible
+        d=Pyro4.Daemon()
+        d.shutdown()
+
+
 class CoreTests(unittest.TestCase):
     def setUp(self):
         Pyro4.config.HMAC_KEY=tobytes("testsuite")
@@ -42,6 +61,35 @@ class CoreTests(unittest.TestCase):
         except AttributeError:
             pass
 
+    def testConfigParseBool(self):
+        config=Pyro4.configuration.Configuration()
+        self.assertTrue(type(config.COMPRESSION) is bool)
+        os.environ["PYRO_COMPRESSION"]="yes"
+        config.refresh()
+        self.assertTrue(config.COMPRESSION)
+        os.environ["PYRO_COMPRESSION"]="off"
+        config.refresh()
+        self.assertFalse(config.COMPRESSION)
+        os.environ["PYRO_COMPRESSION"]="foobar"
+        self.assertRaises(ValueError, config.refresh)
+        del os.environ["PYRO_COMPRESSION"]
+
+    def testConfigDump(self):
+        config=Pyro4.configuration.Configuration()
+        dump=config.dump()
+        self.assertTrue("version:" in dump)
+        self.assertTrue("LOGLEVEL" in dump)
+
+    def testLogInit(self):
+        _=logging.getLogger("Pyro")
+        os.environ["PYRO_LOGLEVEL"]="DEBUG"
+        os.environ["PYRO_LOGFILE"]="{stderr}"
+        reload(Pyro4)
+        _=logging.getLogger("Pyro")
+        del os.environ["PYRO_LOGLEVEL"]
+        del os.environ["PYRO_LOGFILE"]
+        reload(Pyro4)
+
     def testUriStrAndRepr(self):
         uri="PYRONAME:some_obj_name"
         p=Pyro4.core.URI(uri)
@@ -319,23 +367,7 @@ class CoreTests(unittest.TestCase):
         with p:
             self.assertTrue(p._pyroUri is not None)
         with p:
-            self.assertTrue(p._pyroUri is not None)            
-
-    def testRemoteMethod(self):
-        class Proxy(object):
-            def invoke(self, name, args, kwargs):
-                return "INVOKED name=%s args=%s kwargs=%s" % (name,args,kwargs)
-            def __getattr__(self, name):
-                return Pyro4.core._RemoteMethod(self.invoke, name)
-        o=Proxy()
-        self.assertEqual("INVOKED name=foo args=(1,) kwargs={}", o.foo(1)) #normal
-        self.assertEqual("INVOKED name=foo.bar args=(1,) kwargs={}", o.foo.bar(1)) #dotted
-        self.assertEqual("INVOKED name=foo.bar args=(1, 'hello') kwargs={'a': True}", o.foo.bar(1,"hello",a=True))
-        p=Pyro4.core.Proxy("PYRO:obj@host:666")
-        a=p.someattribute
-        self.assertTrue(isinstance(a, Pyro4.core._RemoteMethod), "attribute access should just be a RemoteMethod")
-        a2=a.nestedattribute
-        self.assertTrue(isinstance(a2, Pyro4.core._RemoteMethod), "nested attribute should just be another RemoteMethod")
+            self.assertTrue(p._pyroUri is not None)
 
     def testNoConnect(self):
         wrongUri=Pyro4.core.URI("PYRO:foobar@localhost:59999")
@@ -385,8 +417,88 @@ class CoreTests(unittest.TestCase):
         t=Test()
         self.assertEqual(True, getattr(t.method,"_pyroCallback"))
         self.assertEqual(False, getattr(t.method2,"_pyroCallback", False))
-        
-        
+
+
+class RemoteMethodTests(unittest.TestCase):
+    class BatchProxyMock(object):
+        def _pyroBatch(self):
+            return Pyro4.core._BatchProxy(self.pyroInvokeBatch)
+        def pyroInvokeBatch(self, calls, oneway=False):
+            self.result=[]
+            for name, args, kwargs in calls:
+                if name=="error":
+                    self.result.append(Pyro4.core._ExceptionWrapper(ValueError("some exception")))
+                    break  # stop processing the rest, this is what Pyro should do in case of an error in a batch
+                else:
+                    self.result.append("INVOKED %s args=%s kwargs=%s" % (name,args,kwargs))
+            if oneway:
+                return
+            else:
+                return self.result
+
+    def testRemoteMethod(self):
+        class ProxyMock(object):
+            def invoke(self, name, args, kwargs):
+                return "INVOKED name=%s args=%s kwargs=%s" % (name,args,kwargs)
+            def __getattr__(self, name):
+                return Pyro4.core._RemoteMethod(self.invoke, name)
+        o=ProxyMock()
+        self.assertEqual("INVOKED name=foo args=(1,) kwargs={}", o.foo(1)) #normal
+        self.assertEqual("INVOKED name=foo.bar args=(1,) kwargs={}", o.foo.bar(1)) #dotted
+        self.assertEqual("INVOKED name=foo.bar args=(1, 'hello') kwargs={'a': True}", o.foo.bar(1,"hello",a=True))
+        p=Pyro4.core.Proxy("PYRO:obj@host:666")
+        a=p.someattribute
+        self.assertTrue(isinstance(a, Pyro4.core._RemoteMethod), "attribute access should just be a RemoteMethod")
+        a2=a.nestedattribute
+        self.assertTrue(isinstance(a2, Pyro4.core._RemoteMethod), "nested attribute should just be another RemoteMethod")
+
+    def testBatchMethod(self):
+        proxy=self.BatchProxyMock()
+        batch=proxy._pyroBatch()
+        self.assertEqual(None, batch.foo(42))
+        self.assertEqual(None, batch.bar("abc"))
+        self.assertEqual(None, batch.baz(42,"abc",arg=999))
+        self.assertEqual(None, batch.error())   # generate an exception
+        self.assertEqual(None, batch.foo(42))   # this call should not be performed after the error
+        results=batch()
+        result=next(results)
+        self.assertEqual("INVOKED foo args=(42,) kwargs={}",result)
+        result=next(results)
+        self.assertEqual("INVOKED bar args=('abc',) kwargs={}",result)
+        result=next(results)
+        self.assertEqual("INVOKED baz args=(42, 'abc') kwargs={'arg': 999}",result)
+        self.assertRaises(ValueError, next, results)  # the call to error() should generate an exception
+        self.assertRaises(StopIteration, next, results)   # and now there should not be any more results
+        self.assertEqual(4, len(proxy.result))   # should have done 4 calls, not 5
+
+    def testBatchMethodOneway(self):
+        proxy=self.BatchProxyMock()
+        batch=proxy._pyroBatch()
+        self.assertEqual(None, batch.foo(42))
+        self.assertEqual(None, batch.bar("abc"))
+        self.assertEqual(None, batch.baz(42,"abc",arg=999))
+        self.assertEqual(None, batch.error())   # generate an exception
+        self.assertEqual(None, batch.foo(42))   # this call should not be performed after the error
+        results=batch(oneway=True)
+        self.assertEqual(None, results)          # oneway always returns None
+        self.assertEqual(4, len(proxy.result))   # should have done 4 calls, not 5
+
+    def testBatchMethodReuse(self):
+        proxy=self.BatchProxyMock()
+        batch=proxy._pyroBatch()
+        batch.foo(1)
+        batch.foo(2)
+        results=batch()
+        self.assertEqual(['INVOKED foo args=(1,) kwargs={}', 'INVOKED foo args=(2,) kwargs={}'], list(results))
+        # re-use the batch proxy:
+        batch.foo(3)
+        batch.foo(4)
+        results=batch()
+        self.assertEqual(['INVOKED foo args=(3,) kwargs={}', 'INVOKED foo args=(4,) kwargs={}'], list(results))
+        results=batch()
+        self.assertEqual(0, len(list(results)))
+
+
 if __name__ == "__main__":
     #import sys;sys.argv = ['', 'Test.testName']
     unittest.main()
