@@ -48,16 +48,45 @@ class DaemonLoopThread(threadutil.Thread):
             self.pyrodaemon.requestLoop()
         except:
             print("Swallow exception from terminated daemon")
-        
-class ServerTestsThreadNoTimeout(unittest.TestCase):
-    SERVERTYPE="thread"
-    COMMTIMEOUT=None
+
+
+class DaemonWithSabotagedHandshake(Pyro4.core.Daemon):
+    def handshake(self, conn):
+        # a bit of a hack, overriding this internal method to return a CONNECTFAIL...
+        data=tobytes("rigged connection failure")
+        msg=Pyro4.core.MessageFactory.createMessage(Pyro4.core.MessageFactory.MSG_CONNECTFAIL, data, 0, 1)
+        conn.send(msg)
+        return False
+    
+class ServerTestsBrokenHandshake(unittest.TestCase):
+    def setUp(self):
+        Pyro4.config.HMAC_KEY=tobytes("testsuite")
+        self.daemon=DaemonWithSabotagedHandshake(port=0)
+        obj=MyThing()
+        uri=self.daemon.register(obj, "something")
+        self.objectUri=uri
+        self.daemonthread=DaemonLoopThread(self.daemon)
+        self.daemonthread.start()
+        self.daemonthread.running.wait()
+    def tearDown(self):
+        time.sleep(0.05)
+        self.daemon.shutdown()
+        self.daemonthread.join()
+        Pyro4.config.HMAC_KEY=None
+    def testDaemonConnectFail(self):
+        # check what happens when the daemon responds with a failed connection msg
+        with Pyro4.Proxy(self.objectUri) as p:
+            try:
+                p.ping()
+                self.fail("expected CommunicationError")
+            except Pyro4.errors.CommunicationError:
+                xv=sys.exc_info()[1]
+                message=str(xv)
+                self.assertTrue("rigged connection failure" in message)
+
+class ServerTestsSingle(unittest.TestCase):
+    """tests that are fine to run with just a single server type"""
     def setUp(self):
-        Pyro4.config.POLLTIMEOUT=0.1
-        Pyro4.config.SERVERTYPE=self.SERVERTYPE
-        Pyro4.config.COMMTIMEOUT=self.COMMTIMEOUT
-        Pyro4.config.THREADPOOL_MINTHREADS=2
-        Pyro4.config.THREADPOOL_MAXTHREADS=20
         Pyro4.config.HMAC_KEY=tobytes("testsuite")
         self.daemon=Pyro4.core.Daemon(port=0)
         obj=MyThing()
@@ -70,8 +99,6 @@ class ServerTestsThreadNoTimeout(unittest.TestCase):
         time.sleep(0.05)
         self.daemon.shutdown()
         self.daemonthread.join()
-        Pyro4.config.SERVERTYPE="thread"
-        Pyro4.config.COMMTIMEOUT=None
         Pyro4.config.HMAC_KEY=None
 
     def testNoDottedNames(self):
@@ -97,7 +124,6 @@ class ServerTestsThreadNoTimeout(unittest.TestCase):
                 key=list(result[2].keys())[0]
                 self.assertTrue(key==unichr(0x20ac))
 
-
     def testDottedNames(self):
         try:
             Pyro4.config.DOTTEDNAMES=True
@@ -111,6 +137,51 @@ class ServerTestsThreadNoTimeout(unittest.TestCase):
         finally:
             Pyro4.config.DOTTEDNAMES=False
 
+    def testNormalProxy(self):
+        with Pyro4.core.Proxy(self.objectUri) as p:
+            self.assertEqual(42,p.multiply(7,6))
+
+    def testBatchProxy(self):
+        with Pyro4.core.Proxy(self.objectUri) as p:
+            batch=p._pyroBatch()
+            self.assertEqual(None,batch.multiply(7,6))
+            self.assertEqual(None,batch.divide(999,3))
+            self.assertEqual(None,batch.ping())
+            self.assertEqual(None,batch.divide(999,0))      # force an exception here
+            self.assertEqual(None,batch.multiply(3,4))      # this call should not be performed after the error
+            results=batch()
+            self.assertEqual(42,next(results))
+            self.assertEqual(333,next(results))
+            self.assertEqual(None,next(results))
+            self.assertRaises(ZeroDivisionError, next, results)     # 999//0 should raise this error
+            self.assertRaises(StopIteration, next, results)     # no more results should be available after the error
+
+
+class ServerTestsThreadNoTimeout(unittest.TestCase):
+    SERVERTYPE="thread"
+    COMMTIMEOUT=None
+    def setUp(self):
+        Pyro4.config.POLLTIMEOUT=0.1
+        Pyro4.config.SERVERTYPE=self.SERVERTYPE
+        Pyro4.config.COMMTIMEOUT=self.COMMTIMEOUT
+        Pyro4.config.THREADPOOL_MINTHREADS=2
+        Pyro4.config.THREADPOOL_MAXTHREADS=20
+        Pyro4.config.HMAC_KEY=tobytes("testsuite")
+        self.daemon=Pyro4.core.Daemon(port=0)
+        obj=MyThing()
+        uri=self.daemon.register(obj, "something")
+        self.objectUri=uri
+        self.daemonthread=DaemonLoopThread(self.daemon)
+        self.daemonthread.start()
+        self.daemonthread.running.wait()
+    def tearDown(self):
+        time.sleep(0.05)
+        self.daemon.shutdown()
+        self.daemonthread.join()
+        Pyro4.config.SERVERTYPE="thread"
+        Pyro4.config.COMMTIMEOUT=None
+        Pyro4.config.HMAC_KEY=None
+
     def testConnectionStuff(self):
         p1=Pyro4.core.Proxy(self.objectUri)
         p2=Pyro4.core.Proxy(self.objectUri)
